@@ -339,7 +339,7 @@ async function scheduleMac(name, base, args = "") {
   let build_base_without_command_symbol = merge(mac_base, {
     maxRunTime: 7200,
     artifacts: [{
-      expires: 24 * 7,
+      expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
       type: "directory",
       path: "public"
     }],
@@ -409,7 +409,7 @@ async function scheduleLinux(name, overrides, args = "") {
   let artifacts_and_kind = {
     artifacts: {
       public: {
-        expires: 24 * 7,
+        expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
         type: "directory",
         path: "/home/worker/artifacts"
       }
@@ -635,9 +635,9 @@ async function scheduleLinux(name, overrides, args = "") {
 
 /*****************************************************************************/
 
-function scheduleFuzzingRun(base, name, target, max_len, symbol = null, corpus = null) {
-  const MAX_FUZZ_TIME = 300;
+const MAX_FUZZ_TIME = 300;
 
+function scheduleFuzzingRun(base, name, target, corpus = null) {
   queue.scheduleTask(merge(base, {
     name,
     command: [
@@ -645,12 +645,11 @@ function scheduleFuzzingRun(base, name, target, max_len, symbol = null, corpus =
       "-c",
       "bin/checkout.sh && nss/automation/taskcluster/scripts/fuzz.sh " +
         `${target} ${corpus || target} ` +
-        `-max_total_time=${MAX_FUZZ_TIME} ` +
-        `-max_len=${max_len}`
+        `-max_total_time=${MAX_FUZZ_TIME}`
     ],
     provisioner: "nss-t",
     workerType: "t-linux-xlarge-gcp",
-    symbol: symbol || name
+    symbol: name.replace(" ", "-")
   }));
 }
 
@@ -672,15 +671,9 @@ async function scheduleFuzzing() {
 
   // Build base definition.
   let build_base = merge(base, {
-    command: [
-      "/bin/bash",
-      "-c",
-      "bin/checkout.sh && " +
-      "nss/automation/taskcluster/scripts/build_gyp.sh --fuzz"
-    ],
     artifacts: {
       public: {
-        expires: 24 * 7,
+        expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
         type: "directory",
         path: "/home/worker/artifacts"
       }
@@ -691,13 +684,19 @@ async function scheduleFuzzing() {
 
   // The task that builds NSPR+NSS.
   let task_build = queue.scheduleTask(merge(build_base, {
-    name: "Linux x64 (debug, fuzz)"
+    name: "Linux x64 (debug, fuzz)",
+    command: [
+      "/bin/bash",
+      "-c",
+      "bin/checkout.sh && " +
+      "nss/automation/taskcluster/scripts/build_gyp.sh --fuzz --disable-tests -Ddisable_libpkix=1 && " +
+      "nss/automation/taskcluster/scripts/build_cryptofuzz.sh"
+    ],
   }));
 
   // The task that builds NSPR+NSS (TLS fuzzing mode).
   let task_build_tls = queue.scheduleTask(merge(build_base, {
     name: "Linux x64 (debug, TLS fuzz)",
-    symbol: "B",
     group: "TLS",
     command: [
       "/bin/bash",
@@ -711,6 +710,7 @@ async function scheduleFuzzing() {
   queue.scheduleTask(merge(base, {
     parent: task_build_tls,
     name: "Gtests",
+    group: "TLS",
     command: [
       "/bin/bash",
       "-c",
@@ -725,36 +725,37 @@ async function scheduleFuzzing() {
 
   // Schedule fuzzing runs.
   let run_base = merge(base, {parent: task_build, kind: "test"});
-  scheduleFuzzingRun(run_base, "CertDN", "certDN", 4096);
-  scheduleFuzzingRun(run_base, "QuickDER", "quickder", 10000);
+  scheduleFuzzingRun(run_base, "CertDN", "certDN");
+  scheduleFuzzingRun(run_base, "PKCS7", "pkcs7");
+  scheduleFuzzingRun(run_base, "PKCS8", "pkcs8");
+  scheduleFuzzingRun(run_base, "PKCS12", "pkcs12");
+  scheduleFuzzingRun(run_base, "QuickDER", "quickder");
 
-  // Schedule MPI fuzzing runs.
-  let mpi_base = merge(run_base, {group: "MPI"});
-  let mpi_names = ["add", "addmod", "div", "mod", "mulmod", "sqr",
-                   "sqrmod", "sub", "submod"];
-  for (let name of mpi_names) {
-    scheduleFuzzingRun(mpi_base, `MPI (${name})`, `mpi-${name}`, 4096, name);
-  }
-  scheduleFuzzingRun(mpi_base, `MPI (invmod)`, `mpi-invmod`, 256, "invmod");
-  scheduleFuzzingRun(mpi_base, `MPI (expmod)`, `mpi-expmod`, 2048, "expmod");
+  scheduleFuzzingRun(run_base, "TLS Client", "tls-client", "tls-client-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "TLS Server", "tls-server", "tls-server-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "DTLS Client", "dtls-client", "dtls-client-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "DTLS Server", "dtls-server", "dtls-server-no_fuzzer_mode");
 
-  // Schedule TLS fuzzing runs (non-fuzzing mode).
-  let tls_base = merge(run_base, {group: "TLS"});
-  scheduleFuzzingRun(tls_base, "TLS Client", "tls-client", 20000, "client-nfm",
-                     "tls-client-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "TLS Server", "tls-server", 20000, "server-nfm",
-                     "tls-server-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "DTLS Client", "dtls-client", 20000,
-                     "dtls-client-nfm", "dtls-client-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "DTLS Server", "dtls-server", 20000,
-                     "dtls-server-nfm", "dtls-server-no_fuzzer_mode");
+  // Schedule Cryptofuzz.
+  queue.scheduleTask(merge(run_base, {
+    name: "Cryptofuzz",
+    command: [
+      "/bin/bash",
+      "-c",
+      "bin/checkout.sh && nss/automation/taskcluster/scripts/cryptofuzz.sh " +
+      `-max_total_time=${MAX_FUZZ_TIME}`,
+    ],
+    provisioner: "nss-t",
+    workerType: "t-linux-xlarge-gcp",
+    symbol: "Cryptofuzz",
+  }));
 
   // Schedule TLS fuzzing runs (fuzzing mode).
-  let tls_fm_base = merge(tls_base, {parent: task_build_tls});
-  scheduleFuzzingRun(tls_fm_base, "TLS Client", "tls-client", 20000, "client");
-  scheduleFuzzingRun(tls_fm_base, "TLS Server", "tls-server", 20000, "server");
-  scheduleFuzzingRun(tls_fm_base, "DTLS Client", "dtls-client", 20000, "dtls-client");
-  scheduleFuzzingRun(tls_fm_base, "DTLS Server", "dtls-server", 20000, "dtls-server");
+  let tls_base = merge(run_base, {parent: task_build_tls, group: "TLS"});
+  scheduleFuzzingRun(tls_base, "TLS Client", "tls-client");
+  scheduleFuzzingRun(tls_base, "TLS Server", "tls-server");
+  scheduleFuzzingRun(tls_base, "DTLS Client", "dtls-client");
+  scheduleFuzzingRun(tls_base, "DTLS Server", "dtls-server");
 
   return queue.submit();
 }
@@ -777,15 +778,9 @@ async function scheduleFuzzing32() {
 
   // Build base definition.
   let build_base = merge(base, {
-    command: [
-      "/bin/bash",
-      "-c",
-      "bin/checkout.sh && " +
-      "nss/automation/taskcluster/scripts/build_gyp.sh --fuzz -t ia32"
-    ],
     artifacts: {
       public: {
-        expires: 24 * 7,
+        expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
         type: "directory",
         path: "/home/worker/artifacts"
       }
@@ -796,13 +791,19 @@ async function scheduleFuzzing32() {
 
   // The task that builds NSPR+NSS.
   let task_build = queue.scheduleTask(merge(build_base, {
-    name: "Linux 32 (debug, fuzz)"
+    name: "Linux 32 (debug, fuzz)",
+    command: [
+      "/bin/bash",
+      "-c",
+      "bin/checkout.sh && " +
+      "nss/automation/taskcluster/scripts/build_gyp.sh --fuzz -t ia32 --disable-tests -Ddisable_libpkix=1 && " +
+      "nss/automation/taskcluster/scripts/build_cryptofuzz.sh --i386"
+    ],
   }));
 
   // The task that builds NSPR+NSS (TLS fuzzing mode).
   let task_build_tls = queue.scheduleTask(merge(build_base, {
     name: "Linux 32 (debug, TLS fuzz)",
-    symbol: "B",
     group: "TLS",
     command: [
       "/bin/bash",
@@ -816,6 +817,7 @@ async function scheduleFuzzing32() {
   queue.scheduleTask(merge(base, {
     parent: task_build_tls,
     name: "Gtests",
+    group: "TLS",
     command: [
       "/bin/bash",
       "-c",
@@ -830,35 +832,37 @@ async function scheduleFuzzing32() {
 
   // Schedule fuzzing runs.
   let run_base = merge(base, {parent: task_build, kind: "test"});
-  scheduleFuzzingRun(run_base, "CertDN", "certDN", 4096);
-  scheduleFuzzingRun(run_base, "QuickDER", "quickder", 10000);
+  scheduleFuzzingRun(run_base, "CertDN", "certDN");
+  scheduleFuzzingRun(run_base, "PKCS7", "pkcs7");
+  scheduleFuzzingRun(run_base, "PKCS8", "pkcs8");
+  scheduleFuzzingRun(run_base, "PKCS12", "pkcs12");
+  scheduleFuzzingRun(run_base, "QuickDER", "quickder");
 
-  // Schedule MPI fuzzing runs.
-  let mpi_base = merge(run_base, {group: "MPI"});
-  let mpi_names = ["add", "addmod", "div", "expmod", "mod", "mulmod", "sqr",
-                   "sqrmod", "sub", "submod"];
-  for (let name of mpi_names) {
-    scheduleFuzzingRun(mpi_base, `MPI (${name})`, `mpi-${name}`, 4096, name);
-  }
-  scheduleFuzzingRun(mpi_base, `MPI (invmod)`, `mpi-invmod`, 256, "invmod");
+  scheduleFuzzingRun(run_base, "TLS Client", "tls-client", "tls-client-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "TLS Server", "tls-server", "tls-server-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "DTLS Client", "dtls-client", "dtls-client-no_fuzzer_mode");
+  scheduleFuzzingRun(run_base, "DTLS Server", "dtls-server", "dtls-server-no_fuzzer_mode");
 
-  // Schedule TLS fuzzing runs (non-fuzzing mode).
-  let tls_base = merge(run_base, {group: "TLS"});
-  scheduleFuzzingRun(tls_base, "TLS Client", "tls-client", 20000, "client-nfm",
-                     "tls-client-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "TLS Server", "tls-server", 20000, "server-nfm",
-                     "tls-server-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "DTLS Client", "dtls-client", 20000,
-                     "dtls-client-nfm", "dtls-client-no_fuzzer_mode");
-  scheduleFuzzingRun(tls_base, "DTLS Server", "dtls-server", 20000,
-                     "dtls-server-nfm", "dtls-server-no_fuzzer_mode");
+  // Schedule Cryptofuzz.
+  queue.scheduleTask(merge(run_base, {
+    name: "Cryptofuzz",
+    command: [
+      "/bin/bash",
+      "-c",
+      "bin/checkout.sh && nss/automation/taskcluster/scripts/cryptofuzz.sh " +
+      `-max_total_time=${MAX_FUZZ_TIME}`,
+    ],
+    provisioner: "nss-t",
+    workerType: "t-linux-xlarge-gcp",
+    symbol: "Cryptofuzz",
+  }));
 
   // Schedule TLS fuzzing runs (fuzzing mode).
-  let tls_fm_base = merge(tls_base, {parent: task_build_tls});
-  scheduleFuzzingRun(tls_fm_base, "TLS Client", "tls-client", 20000, "client");
-  scheduleFuzzingRun(tls_fm_base, "TLS Server", "tls-server", 20000, "server");
-  scheduleFuzzingRun(tls_fm_base, "DTLS Client", "dtls-client", 20000, "dtls-client");
-  scheduleFuzzingRun(tls_fm_base, "DTLS Server", "dtls-server", 20000, "dtls-server");
+  let tls_base = merge(run_base, {parent: task_build_tls, group: "TLS"});
+  scheduleFuzzingRun(tls_base, "TLS Client", "tls-client");
+  scheduleFuzzingRun(tls_base, "TLS Server", "tls-server");
+  scheduleFuzzingRun(tls_base, "DTLS Client", "dtls-client");
+  scheduleFuzzingRun(tls_base, "DTLS Server", "dtls-server");
 
   return queue.submit();
 }
@@ -884,7 +888,7 @@ async function scheduleWindows(name, base, build_script) {
 
   let artifacts_and_kind = {
     artifacts: [{
-      expires: 24 * 7,
+      expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
       type: "directory",
       path: "public\\build"
     }],
@@ -1146,7 +1150,7 @@ async function scheduleTools() {
     },
     artifacts: {
       public: {
-        expires: 24 * 7,
+        expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
         type: "directory",
         path: "/home/worker/artifacts"
       }
@@ -1177,7 +1181,7 @@ async function scheduleTools() {
     features: ["allowPtrace"],
     artifacts: {
       public: {
-        expires: 24 * 7,
+        expires: 24 * (process.env.MOZ_SCM_LEVEL == "3" ? 90 : 7),
         type: "directory",
         path: "/home/worker/artifacts"
       }
